;----------------------------------
; PopTray 3.0+ NSIS install script
;----------------------------------

!define VER_MAJOR "3"
!define VER_MINOR "03"

!define MUI_PRODUCT "PopTray"
!define MUI_VERSION "${VER_MAJOR}.${VER_MINOR}"

!include "MUI.nsh"

!define RunOnce $0
!define Language $1
!define Sounds $2
!define StartMenuIcons $3
!define StartupIcon $4
!define DesktopIcon $5

!define TEMP $R9

;------------------------------------------------------------------[ Configuration ]---

OutFile "Deploy\PopTray${VER_MAJOR}${VER_MINOR}.exe"
SetCompressor bzip2

InstType "Full"
InstType "Minimum"

ShowInstDetails show
ShowUninstDetails show

;------------------------------------------------------------------[ Modern UI Configuration ]---
  !define MUI_UNCUSTOMPAGECOMMANDS

  !define MUI_WELCOMEPAGE
  !define MUI_COMPONENTSPAGE
    !define MUI_COMPONENTSPAGE_SMALLDESC
  !define MUI_DIRECTORYPAGE
    
  !define MUI_ABORTWARNING
  
  !define MUI_UNINSTALLER
  ;!define MUI_UNCONFIRMPAGE
  !define MUI_FINISHPAGE
    !define MUI_FINISHPAGE_NOREBOOTSUPPORT
    !define MUI_FINISHPAGE_RUN $INSTDIR\PopTray.exe
    !define MUI_FINISHPAGE_SHOWREADME $INSTDIR\ReadMe.txt
    !define MUI_FINISHPAGE_SHOWREADME_NOTCHECKED
    
  !define MUI_HEADERBITMAP "images\setup_header.bmp"
  !define MUI_SPECIALBITMAP "images\setup.bmp"
  
  !define MUI_ICON "images\Setup.ico"
  !define MUI_UNICON "images\Uninstall.ico"

;------------------------------------------------------------------[ Pages ]---
  
  UninstPage custom un.SetPage
  !insertmacro MUI_UNPAGECOMMAND_INSTFILES
  
;------------------------------------------------------------------[ Languages ]---
 
  !define MUI_TEXT_WELCOME_INFO_TEXT "This wizard will guide you through the installation of PopTray\r\n\r\nPopTray is a full-featured, open-source, e-mail notifier with an easy to use interface.\r\n\r\n"
  !define MUI_TEXT_FINISH_SHOWREADME "Show ReadMe.txt"
  !insertmacro MUI_LANGUAGE "English"
  
;------------------------------------------------------------------[ Language Strings ]---

  LangString DESC_SecPopTray ${LANG_ENGLISH} "Copy PopTray application files to the application folder."
  LangString DESC_SecLang ${LANG_ENGLISH} "Install the Language files for multi-language PopTray."
  LangString DESC_SecSound ${LANG_ENGLISH} "Install example notification sound files."
  LangString DESC_SecIcons ${LANG_ENGLISH} "Select the icons to create."
  LangString DESC_SecStartMenu ${LANG_ENGLISH} "Create a PopTray group under the start menu with icons the the PopTray files."
  LangString DESC_SecStartup ${LANG_ENGLISH} "Automatically start PopTray when Windows starts"
  LangString DESC_SecDesktop ${LANG_ENGLISH} "Put PopTray icon on the Desktop"

;------------------------------------------------------------------[ Reserve Files ]---
  
  ReserveFile "${MUI_HEADERBITMAP}"
  ReserveFile "${MUI_SPECIALBITMAP}"
  ReserveFile "NSIS.ini" 

;------------------------------------------------------------------[ Installer Sections ]---

Section "PopTray (required)" SecPopTray
  SectionIn 1 2

  SetOutPath "$INSTDIR"

  !define HWND $R0
  !define Count $R1

  ; try copying EXE.  If it fails try killing the running PopTray upto 5 times
  SetOverwrite try
  TryPoptray:
 	IntOp ${Count} ${Count} + 1
        ;DetailPrint ${Count}
  	IntCmp ${Count} 5 CantClose
  	ClearErrors
  	File "PopTray.exe"
  	IfErrors ClosePopTray OtherFiles
  ClosePopTray:
  	; close running PopTray
  	FindWindow ${HWND} "TfrmPopMain"
  	IntCmp ${Count} 1 Print Send Send
  Print:
  	DetailPrint "Closing existing PopTray"
  Send:
  	SendMessage ${HWND} 1036 0 0 ; UM_QUIT = 1036
  	;SendMessage ${HWND} ${WM_CLOSE} 0 0
  	Sleep 1000
  	Goto TryPopTray
  CantClose:
  	DetailPrint "Can't close existing PopTray"
  	SetOverwrite on
  	File "PopTray.exe"
  
  ; rest of the files
  OtherFiles:
  	SetOverwrite on
  	File "PopTray.chm"
  	File "Readme.txt"
  	File "History.txt"
  
  ;MessageBox MB_OK "Debug Wait"

  WriteRegStr HKLM "Software\${MUI_PRODUCT}" "" $INSTDIR
  WriteRegStr HKLM "Software\${MUI_PRODUCT}" "Language" 0
  WriteRegStr HKLM "Software\${MUI_PRODUCT}" "Sounds" 0
  WriteRegStr HKLM "Software\${MUI_PRODUCT}" "StartMenuIcons" 0
  WriteRegStr HKLM "Software\${MUI_PRODUCT}" "StartupIcon" 0
  WriteRegStr HKLM "Software\${MUI_PRODUCT}" "DesktopIcon" 0
  
  ; uninstall data
  WriteRegExpandStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "UninstallString" "$INSTDIR\Uninstall.exe"
  WriteRegExpandStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "InstallLocation" "$INSTDIR"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "DisplayName" "${MUI_PRODUCT} ${MUI_VERSION}"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "DisplayIcon" "$INSTDIR\PopTray.exe,0"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "DisplayVersion" "${MUI_VERSION}"
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "VersionMajor" "${VER_MAJOR}"
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "VersionMinor" "${VER_MINOR}"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "Publisher" "Renier Crause"
  WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "URLInfoAbout" "http://www.poptray.org"
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "NoModify" "1"
  WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}" "NoRepair" "1"
  
  ; delete inno setup files
  Delete "$INSTDIR\unins000.dat"
  Delete "$INSTDIR\unins000.exe"
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\PopTray_is1"
  
  ; uninstaller
  Delete "$INSTDIR\Uninstall.exe"
  WriteUninstaller "$INSTDIR\Uninstall.exe"
SectionEnd

SubSection "Icons" SecIcons

Section "Start Menu Icons" SecStartMenu
  SectionIn 1 2
  WriteRegStr HKLM "Software\${MUI_PRODUCT}" "StartMenuIcons" 1
  CreateDirectory $SMPROGRAMS\PopTray
  CreateShortCut "$SMPROGRAMS\PopTray\PopTray.lnk" "$INSTDIR\PopTray.exe" ""
  CreateShortCut "$SMPROGRAMS\PopTray\Uninstall.lnk" "$INSTDIR\Uninstall.exe" ""
  CreateShortCut "$SMPROGRAMS\PopTray\PopTray Help.lnk" "$INSTDIR\PopTray.chm" ""
  CreateShortCut "$SMPROGRAMS\PopTray\Read Me.lnk" "$INSTDIR\ReadMe.txt" ""
  CreateShortCut "$SMPROGRAMS\PopTray\Version History.lnk" "$INSTDIR\History.txt" ""
SectionEnd

Section "Startup Icon" SecStartup
  SectionIn 1
  WriteRegStr HKLM "Software\${MUI_PRODUCT}" "StartupIcon" 1
  CreateShortCut "$SMSTARTUP\PopTray.lnk" "$INSTDIR\PopTray.exe" ""
SectionEnd

Section "Desktop Icon" SecDesktop
  SectionIn 1
  WriteRegStr HKLM "Software\${MUI_PRODUCT}" "DesktopIcon" 1
  CreateShortCut "$DESKTOP\PopTray.lnk" "$INSTDIR\PopTray.exe" ""
SectionEnd

SubSectionEnd

Section "Language files" SecLang
  SectionIn 1
  WriteRegStr HKLM "Software\${MUI_PRODUCT}" "Language" 1
  
  SetOutPath "$INSTDIR\Languages"
  
  ; delete old
  Delete "$INSTDIR\blank.ptlang"
  Delete "$INSTDIR\language.ptlang"
  Delete "$INSTDIR\Afrikaans.ptlang"
  Delete "$INSTDIR\Arabic.ptlang"
  Delete "$INSTDIR\Bulgarian.ptlang"
  Delete "$INSTDIR\Catalan.ptlang"
  Delete "$INSTDIR\Chinese.ptlang"
  Delete "$INSTDIR\Czech.ptlang"
  Delete "$INSTDIR\Danish.ptlang"
  Delete "$INSTDIR\Dutch.ptlang"
  Delete "$INSTDIR\Finnish.ptlang"
  Delete "$INSTDIR\French.ptlang"
  Delete "$INSTDIR\French (Canada).ptlang"
  Delete "$INSTDIR\German.ptlang"
  Delete "$INSTDIR\Hebrew.ptlang"
  Delete "$INSTDIR\Hungarian.ptlang"
  Delete "$INSTDIR\Italian.ptlang"
  Delete "$INSTDIR\Korean.ptlang"
  Delete "$INSTDIR\Norwegian.ptlang"
  Delete "$INSTDIR\Polish.ptlang"
  Delete "$INSTDIR\Portuguese.ptlang"
  Delete "$INSTDIR\Portuguese (Brazil).ptlang"
  Delete "$INSTDIR\Romanian.ptlang"
  Delete "$INSTDIR\Russian.ptlang"
  Delete "$INSTDIR\Slovak.ptlang"
  Delete "$INSTDIR\Spanish.ptlang"
  Delete "$INSTDIR\Spanish (Chile).ptlang"
  Delete "$INSTDIR\Spanish (Latin America).ptlang"
  Delete "$INSTDIR\Spanish (Hispanic America).ptlang"
  Delete "$INSTDIR\Swedish.ptlang"
  Delete "$INSTDIR\Taiwan.ptlang"
  Delete "$INSTDIR\Turkish.ptlang"
  Delete "$INSTDIR\Ukranian.ptlang"
  
  ; copy new
  File "languages\blank.ptlang"
  File "languages\language.ptlang"
  File "languages\Afrikaans.ptlang"
  File "languages\Arabic.ptlang"
  File "languages\Bulgarian.ptlang"
  File "languages\Catalan.ptlang"
  File "languages\Chinese.ptlang"
  File "languages\Czech.ptlang"
  File "languages\Danish.ptlang"
  File "languages\Dutch.ptlang"
  File "languages\Estonian.ptlang"
  File "languages\Finnish.ptlang"
  File "languages\French.ptlang"
  File "languages\French (Canada).ptlang"
  File "languages\German.ptlang"
  File "languages\Hebrew.ptlang"
  File "languages\Hungarian.ptlang"
  File "languages\Italian.ptlang"
  File "languages\Korean.ptlang"
  File "languages\Norwegian.ptlang"
  File "languages\Polish.ptlang"
  File "languages\Portuguese.ptlang"
  File "languages\Portuguese (Brazil).ptlang"
  File "languages\Romanian.ptlang"
  File "languages\Russian.ptlang"
  File "languages\Serbian.ptlang"
  File "languages\Slovak.ptlang"
  File "languages\Slovene.ptlang"
  File "languages\Spanish.ptlang"
  File "languages\Spanish (Chile).ptlang"
  File "languages\Spanish (Hispanic America).ptlang"
  File "languages\Swedish.ptlang"
  File "languages\Taiwan.ptlang"
  File "languages\Turkish.ptlang"
  File "languages\Ukranian.ptlang"
SectionEnd

Section "Sound files" SecSound
  SectionIn 1
  WriteRegStr HKLM "Software\${MUI_PRODUCT}" "Sounds" 1
  
  SetOutPath "$INSTDIR\Sounds"
  
  File "sounds\poptray_newmail_lo.wav"
  File "sounds\poptray_urgent_lo.wav"
  File "sounds\poptray_friend_lo.wav"
  File "sounds\poptray_business_lo.wav"
  File "sounds\poptray_junkmail_lo.wav"
  File "sounds\poptray_spam_lo.wav"
SectionEnd


;!insertmacro MUI_SECTIONS_FINISHHEADER

;------------------------------------------------------------------[ Descriptions ]---

!insertmacro MUI_FUNCTIONS_DESCRIPTION_BEGIN
  !insertmacro MUI_DESCRIPTION_TEXT ${SecPopTray} $(DESC_SecPopTray)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecLang} $(DESC_SecLang)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecSound} $(DESC_SecSound)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecIcons} $(DESC_SecIcons)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecStartMenu} $(DESC_SecStartMenu)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecStartup} $(DESC_SecStartup)
  !insertmacro MUI_DESCRIPTION_TEXT ${SecDesktop} $(DESC_SecDesktop)
!insertmacro MUI_FUNCTIONS_DESCRIPTION_END
 
;------------------------------------------------------------------[ Uninstaller Section ]---

Section "Uninstall"
  ; application files
  Delete "$INSTDIR\PopTray.exe"
  Delete "$INSTDIR\Uninstall.exe"
  Delete "$INSTDIR\PopTray.chm"
  Delete "$INSTDIR\Readme.txt"
  Delete "$INSTDIR\History.txt"
  
  ; start menu icons
  Delete "$SMPROGRAMS\PopTray\PopTray.lnk"
  Delete "$SMPROGRAMS\PopTray\Uninstall.lnk"
  Delete "$SMPROGRAMS\PopTray\PopTray Help.lnk"
  Delete "$SMPROGRAMS\PopTray\PopTray Help.lnk"
  Delete "$SMPROGRAMS\PopTray\Read Me.lnk"
  Delete "$SMPROGRAMS\PopTray\Version History.lnk"
  
  ; other icons
  Delete "$SMSTARTUP\PopTray.lnk"
  Delete "$DESKTOP\PopTray.lnk"
  
  ; sounds
  Delete "$INSTDIR\Sounds\poptray_newmail_lo.wav"
  Delete "$INSTDIR\Sounds\poptray_urgent_lo.wav"
  Delete "$INSTDIR\Sounds\poptray_friend_lo.wav"
  Delete "$INSTDIR\Sounds\poptray_business_lo.wav"
  Delete "$INSTDIR\Sounds\poptray_junkmail_lo.wav"
  Delete "$INSTDIR\Sounds\poptray_spam_lo.wav"

  ; language
  Delete "$INSTDIR\Languages\blank.ptlang"
  Delete "$INSTDIR\Languages\language.ptlang"
  Delete "$INSTDIR\Languages\Afrikaans.ptlang"
  Delete "$INSTDIR\Languages\Arabic.ptlang"
  Delete "$INSTDIR\Languages\Bulgarian.ptlang"
  Delete "$INSTDIR\Languages\Catalan.ptlang"
  Delete "$INSTDIR\Languages\Chinese.ptlang"
  Delete "$INSTDIR\Languages\Czech.ptlang"
  Delete "$INSTDIR\Languages\Danish.ptlang"
  Delete "$INSTDIR\Languages\Dutch.ptlang"
  Delete "$INSTDIR\Languages\Estonian.ptlang"
  Delete "$INSTDIR\Languages\Finnish.ptlang"
  Delete "$INSTDIR\Languages\French.ptlang"
  Delete "$INSTDIR\Languages\French (Canada).ptlang"
  Delete "$INSTDIR\Languages\German.ptlang"
  Delete "$INSTDIR\Languages\Hebrew.ptlang"
  Delete "$INSTDIR\Languages\Hungarian.ptlang"
  Delete "$INSTDIR\Languages\Italian.ptlang"
  Delete "$INSTDIR\Languages\Korean.ptlang"
  Delete "$INSTDIR\Languages\Norwegian.ptlang"
  Delete "$INSTDIR\Languages\Polish.ptlang"
  Delete "$INSTDIR\Languages\Portuguese.ptlang"
  Delete "$INSTDIR\Languages\Portuguese (Brazil).ptlang"
  Delete "$INSTDIR\Languages\Romanian.ptlang"
  Delete "$INSTDIR\Languages\Russian.ptlang"
  Delete "$INSTDIR\Languages\Serbian.ptlang"
  Delete "$INSTDIR\Languages\Slovak.ptlang"
  Delete "$INSTDIR\Languages\Slovene.ptlang"
  Delete "$INSTDIR\Languages\Spanish.ptlang"
  Delete "$INSTDIR\Languages\Spanish (Hispanic America).ptlang"
  Delete "$INSTDIR\Languages\Spanish (Chile).ptlang"
  Delete "$INSTDIR\Languages\Swedish.ptlang"
  Delete "$INSTDIR\Languages\Taiwan.ptlang"
  Delete "$INSTDIR\Languages\Turkish.ptlang"
  Delete "$INSTDIR\Languages\Ukranian.ptlang"
  
  ; configuration files
  !insertmacro MUI_INSTALLOPTIONS_READ ${TEMP} "NSIS.ini" "Field 4" "State"
  StrCmp ${TEMP} "1" "" NoConfig
    Delete "$INSTDIR\PopTray.ini"
    Delete "$INSTDIR\Rules.ini"
    Delete "$INSTDIR\PopTray.customize"
    Delete "$INSTDIR\Preview.customize"
    Delete "$INSTDIR\WhiteList.ptdat"
    Delete "$INSTDIR\BlackList.ptdat"
    Delete "$INSTDIR\Account_*.ids"
  NoConfig:

  ; directories
  RMDir "$INSTDIR\Languages"
  RMDir "$INSTDIR\Sounds"
  RMDir "$INSTDIR"
  RMDir "$SMPROGRAMS\PopTray"

  ; registry
  DeleteRegKey HKCU "Software\${MUI_PRODUCT}"
  DeleteRegKey HKLM "Software\${MUI_PRODUCT}"
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\${MUI_PRODUCT}"
  
  !insertmacro MUI_UNFINISHHEADER
SectionEnd


;------------------------------------------------------------------[ Functions ]---

Function un.SetPage
  !insertmacro MUI_HEADER_TEXT "Uninstall PopTray" "Remove PopTray from you computer."
  !insertmacro MUI_INSTALLOPTIONS_DISPLAY "NSIS.ini"
FunctionEnd


;------------------------------------------------------------------[ Events ]---

Function .onInit
  ; get install dir
  ReadRegStr ${TEMP} HKLM "Software\${MUI_PRODUCT}" ""
  StrCmp ${TEMP} "" +2
    Goto DirFound
    
  ReadRegStr ${TEMP} HKCU "Software\${MUI_PRODUCT}" ""
  StrCmp ${TEMP} "" +2
    Goto DirFound
  
  ReadRegStr ${TEMP} HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\PopTray_is1" "Inno Setup: App Path"
  StrCmp ${TEMP} "" +2
    Goto DirFound
  
  ;Else:
    StrCpy ${TEMP} "$PROGRAMFILES\${MUI_PRODUCT}"
  DirFound:
    StrCpy $INSTDIR ${TEMP}

  ; read selections from registry
  ReadRegStr ${StartMenuIcons} HKLM "Software\${MUI_PRODUCT}" "StartMenuIcons"
  ReadRegStr ${StartupIcon} HKLM "Software\${MUI_PRODUCT}" "StartupIcon"
  ReadRegStr ${DesktopIcon} HKLM "Software\${MUI_PRODUCT}" "DesktopIcon"
  ReadRegStr ${Language} HKLM "Software\${MUI_PRODUCT}" "Language"
  ReadRegStr ${Sounds} HKLM "Software\${MUI_PRODUCT}" "Sounds"
FunctionEnd

Function un.onInit
  !insertmacro MUI_INSTALLOPTIONS_EXTRACT "NSIS.ini"
  !insertmacro MUI_INSTALLOPTIONS_WRITE "NSIS.ini" "Field 3" "State" "$INSTDIR"
FunctionEnd

Function .onSelChange
  StrCmp ${RunOnce} "Yes" End
    StrCpy ${RunOnce} "Yes"
    SectionSetFlags ${SecPopTray} 9
    ; set icons from registry values
    StrCmp ${StartMenuIcons} "0" "" +2
      SectionSetFlags ${SecStartMenu} 0
    StrCmp ${StartupIcon} "0" "" +2
      SectionSetFlags ${SecStartup} 0
    StrCmp ${DesktopIcon} "0" "" +2
      SectionSetFlags ${SecDesktop} 0
    StrCmp ${Language} "0" "" +2
      SectionSetFlags ${SecLang} 0
    StrCmp ${Sounds} "0" "" +2
      SectionSetFlags ${SecSound} 0
  End:
FunctionEnd
